## (c)2018 Microsoft Corporation.  All rights reserved.  This document is provided "as-is." Information and views expressed in this document,
## including URL and other Internet Web site references, may change without notice. You bear the risk of using it. 
 
## This document does not provide you with any legal rights to any intellectual property in any Microsoft product.
## You may copy and use this document for your internal, reference purposes. You may modify this document
Param (
    [ValidateSet("LyncServer2010","LyncServer2013","Skype4B","SFB2019")] 
    [String]
    $Version = $(throw "Product version is required. Specify LyncServer2010, LyncServer2013, Skype4B or SFB2019")
,
    [Parameter(Mandatory=$false)]
    [string] 
    $Computer
,
    [Parameter(Mandatory=$false)]
    [string] 
    $SQLInstance
)

$filename = "KHI.config"

Function CreateDataCollector($Computer)
{
    Write-Host -ForegroundColor Green "Creating KHI Data Collector on" $Computer"."

    $logmanBin = $env:SystemRoot + "\System32\logman.exe"
    $logmanSyntax = " create counter KHI -o C:\PerfLogs\Admin\KHI_$Computer -s $Computer -f csv -si 15 -v mmddhhmm -max 500 -cf .\KHI.config"
    $cmdLine = $logmanBin + $logmanSyntax

    Invoke-Expression $cmdLine
}

Function CreateLyncServer2010KHIConfigFile
{
    New-Item .\$filename -type file -Force | Out-Null

    #System Performance Counters
    Add-Content .\$filename "`"\Processor Information(*)\% Processor Time`""
    Add-Content .\$filename "`"\Memory\Available Mbytes`""
    Add-Content .\$filename "`"\Network Interface(*)\Output Queue Length`""
    Add-Content .\$filename "`"\Network Interface(*)\Outbound Packets Discarded`""
    Add-Content .\$filename "`"\Network Interface(*)\Inbound Packets Discarded`""
    Add-Content .\$filename "`"\Network Interface(*)\Packets Outbound Discarded`""
    Add-Content .\$filename "`"\Network Interface(*)\Packets Received Discarded`""
    Add-Content .\$filename "`"\PhysicalDisk(*)\Avg. Disk sec/Read`""
    Add-Content .\$filename "`"\PhysicalDisk(*)\Avg. Disk sec/Write`""

    #Front-End Server Counters
    Add-Content .\$filename "`"\LS:USrv - 01 - DBStore\USrv - 002 - Queue Latency (msec)`""
    Add-Content .\$filename "`"\LS:USrv - 01 - DBStore\USrv - 020 - Throttled requests/sec`""
    Add-Content .\$filename "`"\LS:USrv - 01 - DBStore\USrv - 004 - Sproc Latency (msec)`""
    Add-Content .\$filename "`"\LS:SIP - 01 - Peers(*)\SIP - 019 - Sends Timed-Out/sec`""
    Add-Content .\$filename "`"\LS:SIP - 01 - Peers(*)\SIP - 020 - Average outgoing Queue Delay`""
    Add-Content .\$filename "`"\LS:SIP - 01 - Peers(*)\SIP - 023 - Flow-controlled Connections`""
    Add-Content .\$filename "`"\LS:SIP - 02 - Protocol\SIP - 005 - Incoming Requests Dropped/sec`""
    Add-Content .\$filename "`"\LS:SIP - 02 - Protocol\SIP - 009 - Incoming Responses Dropped/sec`""
    Add-Content .\$filename "`"\LS:SIP - 02 - Protocol\SIP - 021 - Average Incoming Message Processing Time`""
    Add-Content .\$filename "`"\LS:SIP - 04 - Responses\SIP - 055 - Local 503 Responses/sec`""
    Add-Content .\$filename "`"\LS:SIP - 07 - Load Management\SIP - 000 - Average Holding Time For Incoming Messages`""
    Add-Content .\$filename "`"\LS:SIP - 07 - Load Management\SIP - 006 - Incoming Messages Timed out`""

    #Logman will ignore Counters that don't exist on host, so we'll try BE with common instance names
    #SQL Server Counters
    Add-Content .\$filename "`"\MSSQL`$RTC:Buffer Manager\Page life expectancy`""
    Add-Content .\$filename "`"\MSSQL`$RTCLOCAL:Buffer Manager\Page life expectancy`""
    Add-Content .\$filename "`"\MSSQL`$MSSQLSERVER:Buffer Manager\Page life expectancy`""

    #Front-End MCU Counters
    Add-Content .\$filename "`"\LS:DATAMCU - 04 - MCU Health And Performance\DATAMCU - 005 - MCU Health State`""
    Add-Content .\$filename "`"\LS:AVMCU - 04 - MCU Health And Performance\AVMCU - 005 - MCU Health State`""
    Add-Content .\$filename "`"\LS:ASMCU - 01 - MCU Health And Performance\ASMCU - 005 - MCU Health State`""
    Add-Content .\$filename "`"\LS:IMMCU - 02 - MCU Health And Performance\IMMCU - 005 - MCU Health State`""
    Add-Content .\$filename "`"\LS:IMMCU - 00 - IMMcu Conferences\IMMCU - 020 - Throttled Sip Connections`""

    #Front-End Web Services Counters
    Add-Content .\$filename "`"\ASP.NET Apps v2.0.50727(*)\Requests Rejected`""
    Add-Content .\$filename "`"\LS:WEB - 00 - Distribution List Expansion\WEB - 011 - Timed out Active Directory Requests/sec`""
    Add-Content .\$filename "`"\LS:WEB - 01 - Address Book File Download\WEB - 005 - Failed File Requests/Second`""
    Add-Content .\$filename "`"\LS:WEB - 02 - Address Book Web Query\WEB - 005 - Failed search requests/sec`""

    #Front-End Conferencing Services Counters
    Add-Content .\$filename "`"\LS:CAA - 00 - Operations\CAA - 000 - Incomplete calls per sec`""
    Add-Content .\$filename "`"\LS:USrv - 25 - Conference Control Notification\USrv - 004 - Notifications in processing`""
    Add-Content .\$filename "`"\LS:USrv - 26 - Conference Mcu Allocator\USrv - 019 - Create Conference Latency (msec)`""

    #Monitoring Server Counters
    Add-Content .\$filename "`"\LS:CDR Service - 00 - DBCdr\CDR Service - 002 - Queue Latency (msec)`""
    Add-Content .\$filename "`"\LS:CDR Service - 00 - DBCdr\CDR Service - 004 - Sproc Latency (msec)`""
    Add-Content .\$filename "`"\LS:CDR Service - 01 - READ\CDR Service - 006 - Dropped messages from MQ`""
    Add-Content .\$filename "`"\LS:Arch Service - 01 - READ\Arch Service - 002 - Messages that failed validation`""
    Add-Content .\$filename "`"\LS:Arch Service - 01 - READ\Arch Service - 006 - Dropped messages from MQ`""
    Add-Content .\$filename "`"\LS:Arch Service - 02 - WRITE\Arch Service - 002 - Messages failed to be written to DB`""

    #Mediation Server Counters
    Add-Content .\$filename "`"\LS:MediationServer - 02 - Media Relay\- 001 - Media Connectivity Check Failure`""
    Add-Content .\$filename "`"\LS:MediationServer - 03 - Health Indices\- 000 - Load Call Failure Index`""
    Add-Content .\$filename "`"\LS:MediationServer - 04 - Global Counters\- 001 - Total failed calls caused by unexpected interaction from the Proxy`""
    Add-Content .\$filename "`"\LS:MediationServer - 05 - Global Per Gateway Counters(*)\- 000 - Total failed calls caused by unexpected interaction from a gateway`""

    #MCX Counters
    Add-Content .\$filename "`"\LS:Mcx - 00 - Mobile Communication Service\MCX - 031 - Push Notification Requests Failed/Second`""
    Add-Content .\$filename "`"\LS:Mcx - 00 - Mobile Communication Service\MCX - 029 - Push Notification Requests Throttled/Second`""
    Add-Content .\$filename "`"\LS:Mcx - 00 - Mobile Communication Service\MCX - 011 - Requests Failed/Second`""
    Add-Content .\$filename "`"\LS:Mcx - 00 - Mobile Communication Service\MCX - 015 - Requests Rejected/Second`""

    #Edge Server Counters
    Add-Content .\$filename "`"\LS:A/V Edge - UDP Counters(*)\A/V Edge - Authentication Failures/sec`""
    Add-Content .\$filename "`"\LS:A/V Edge - UDP Counters(*)\A/V Edge - Allocate Requests Exceeding Port Limit`""
    Add-Content .\$filename "`"\LS:A/V Edge - UDP Counters(*)\A/V Edge - Packets Dropped/sec`""
    Add-Content .\$filename "`"\LS:A/V Edge - TCP Counters(*)\A/V Edge - Authentication Failures/sec`""
    Add-Content .\$filename "`"\LS:A/V Edge - TCP Counters(*)\A/V Edge - Allocate Requests Exceeding Port Limit`""
    Add-Content .\$filename "`"\LS:A/V Edge - TCP Counters(*)\A/V Edge - Packets Dropped/sec`""
    Add-Content .\$filename "`"\LS:SIP - Peers(*)\SIP - Above Limit Connections Dropped (Access Proxies only)`""
    Add-Content .\$filename "`"\LS:DATAPROXY - Server Connections(*)\DATAPROXY - Current count of server connections that are throttled`""
    Add-Content .\$filename "`"\LS:DATAPROXY - Server Connections(*)\DATAPROXY - System is throttling`""
}

Function CreateLyncServer2013KHIConfigFile
{
    New-Item .\$filename -type file -Force | Out-Null
	
	#System Performance Counters
    Add-Content .\$filename "`"\Processor Information(*)\% Processor Time`""
    Add-Content .\$filename "`"\Memory\Available Mbytes`""
    Add-Content .\$filename "`"\Network Interface(*)\Output Queue Length`""
    Add-Content .\$filename "`"\Network Interface(*)\Outbound Packets Discarded`""
    Add-Content .\$filename "`"\Network Interface(*)\Inbound Packets Discarded`""
    Add-Content .\$filename "`"\Network Interface(*)\Packets Outbound Discarded`""
    Add-Content .\$filename "`"\Network Interface(*)\Packets Received Discarded`""
    Add-Content .\$filename "`"\PhysicalDisk(*)\Avg. Disk sec/Read`""
    Add-Content .\$filename "`"\PhysicalDisk(*)\Avg. Disk sec/Write`""

    #Front-End Server Counters
    Add-Content .\$filename "`"\LS:Usrv - DBStore\Usrv - Queue Latency (msec)`""
    Add-Content .\$filename "`"\LS:Usrv - DBStore\Usrv - Sproc Latency (msec)`""
    Add-Content .\$filename "`"\LS:Usrv - DBStore\Usrv - Throttled requests/sec`""
    Add-Content .\$filename "`"\LS:Usrv - REGDBStore\Usrv - Queue Latency (msec)`""
    Add-Content .\$filename "`"\LS:Usrv - REGDBStore\Usrv - Sproc Latency (msec)`""
    Add-Content .\$filename "`"\LS:Usrv - REGDBStore\Usrv - Throttled requests/sec`""
    Add-Content .\$filename "`"\LS:Usrv - SharedDBStore\Usrv - Queue Latency (msec)`""
    Add-Content .\$filename "`"\LS:Usrv - SharedDBStore\Usrv - Sproc Latency (msec)`""
    Add-Content .\$filename "`"\LS:Usrv - SharedDBStore\Usrv - Throttled requests/sec`""
    Add-Content .\$filename "`"\LS:SIP - Peers(*)\SIP - Sends Timed-Out/sec`""
    Add-Content .\$filename "`"\LS:SIP - Responses\SIP - Local 503 Responses/sec`""
    Add-Content .\$filename "`"\LS:SIP - Peers(*)\SIP - Average outgoing Queue Delay`""
    Add-Content .\$filename "`"\LS:SIP - Protocol\SIP - Incoming Requests Dropped/sec`""
    Add-Content .\$filename "`"\LS:SIP - Peers(*)\SIP - Flow-controlled Connections`""
    Add-Content .\$filename "`"\LS:SIP - Protocol\SIP - Incoming Responses Dropped/sec`""
    Add-Content .\$filename "`"\LS:SIP - Protocol\SIP - Average Incoming Message Processing Time`""
    Add-Content .\$filename "`"\LS:SIP - Load Management\SIP - Average Holding Time For Incoming Messages`""
    Add-Content .\$filename "`"\LS:SIP - Load Management\SIP - Incoming Messages Timed out`""
    Add-Content .\$filename "`"\LS:Usrv - Cluster Manager\Usrv - Number of failures of replication operations sent to other Replicas per second`""
    Add-Content .\$filename "`"\LS:XMPPFederation - SIP Instant Messaging\XMPPFederation - Failure IMDNs sent/sec`""
    Add-Content .\$filename "`"\LS:RoutingApps - Emergency Call Routing\RoutingApps - Number of incoming failure responses`""
    Add-Content .\$filename "`"\LS:LYSS - Storage Service API\LYSS - Current percentage of space used by Storage Service DB.`""

    #Logman will ignore Counters that don't exist on host, so we'll try BE with common instance names and FE instance names
    #SQL Server Counters
    Add-Content .\$filename "`"\MSSQL`$RTC:Buffer Manager\Page life expectancy`""
    Add-Content .\$filename "`"\MSSQL`$MSSQLSERVER:Buffer Manager\Page life expectancy`""
    Add-Content .\$filename "`"\MSSQL`$RTCLOCAL:Buffer Manager\Page life expectancy`""
    Add-Content .\$filename "`"\MSSQL`$LYNCLOCAL:Buffer Manager\Page life expectancy`""

    #Front-End MCU Counters
    Add-Content .\$filename "`"\LS:DATAMCU - MCU Health And Performance\DATAMCU - MCU Health State`""
    Add-Content .\$filename "`"\LS:ASMCU - MCU Health And Performance\ASMCU - MCU Health State`""
    Add-Content .\$filename "`"\LS:AVMCU - MCU Health And Performance\AVMCU - MCU Health State`""
    Add-Content .\$filename "`"\LS:IMMCU - MCU Health And Performance\IMMCU - MCU Health State`""
    Add-Content .\$filename "`"\LS:IMMCU - IMMcu Conferences\IMMCU - Throttled Sip Connections`""

    #Front-End Web Services Counters
    Add-Content .\$filename "`"\ASP.NET Apps v4.0.30319(*)\Requests Rejected`""
    Add-Content .\$filename "`"\LS:WEB - Distribution List Expansion\WEB - Timed out Active Directory Requests/sec`""
    Add-Content .\$filename "`"\LS:WEB - Address Book File Download\WEB - Failed File Requests/Second`""
    Add-Content .\$filename "`"\LS:JoinLauncher - Join Launcher Service Failures\JOINLAUNCHER - Join Failures`""
    Add-Content .\$filename "`"\LS:WEB - UCWA\UCWA - HTTP 5xx Responses/Second`""
    Add-Content .\$filename "`"\LS:WEB - Auth Provider related calls\WEB - Failed validate cert calls to the cert auth provider`""
    Add-Content .\$filename "`"\LS:WEB - Address Book Web Query\WEB - Failed search requests/sec`""

    #Front-End Conferencing Services Counters
    Add-Content .\$filename "`"\LS:CAA - Operations\CAA - Incomplete calls per sec`""
    Add-Content .\$filename "`"\LS:USrv - Conference Control Notification\USrv - Notifications in processing`""
    Add-Content .\$filename "`"\LS:USrv - Conference Mcu Allocator\USrv - Factory Call Latency (msec)`""
    Add-Content .\$filename "`"\LS:USrv - Conference Mcu Allocator\USrv - Create Conference Latency (msec)`""
    Add-Content .\$filename "`"\LS:USrv - Conference Mcu Allocator\USrv - Allocation Latency (msec)`""

    #Mediation Server Counters
    Add-Content .\$filename "`"\LS:MediationServer - Media Relay\- Media Connectivity Check Failure`""
    Add-Content .\$filename "`"\LS:MediationServer - Health Indices\- Load Call Failure Index`""
    Add-Content .\$filename "`"\LS:MediationServer - Global Counters\- Total failed calls caused by unexpected interaction from the Proxy`""
    Add-Content .\$filename "`"\LS:MediationServer - Global Per Gateway Counters(*)\- Total failed calls caused by unexpected interaction from a gateway`""

    #Edge Server Counters
    Add-Content .\$filename "`"\LS:A/V Edge - UDP Counters(*)\A/V Edge - Authentication Failures/sec`""
    Add-Content .\$filename "`"\LS:A/V Edge - UDP Counters(*)\A/V Edge - Allocate Requests Exceeding Port Limit`""
    Add-Content .\$filename "`"\LS:A/V Edge - UDP Counters(*)\A/V Edge - Packets Dropped/sec`""
    Add-Content .\$filename "`"\LS:A/V Edge - TCP Counters(*)\A/V Edge - Authentication Failures/sec`""
    Add-Content .\$filename "`"\LS:A/V Edge - TCP Counters(*)\A/V Edge - Allocate Requests Exceeding Port Limit`""
    Add-Content .\$filename "`"\LS:A/V Edge - TCP Counters(*)\A/V Edge - Packets Dropped/sec`""
    Add-Content .\$filename "`"\LS:SIP - Peers(*)\SIP - Above Limit Connections Dropped (Access Proxies only)`""
    Add-Content .\$filename "`"\LS:DATAPROXY - Server Connections(*)\DATAPROXY - Current count of server connections that are throttled`""
    Add-Content .\$filename "`"\LS:DATAPROXY - Server Connections(*)\DATAPROXY - System is throttling`""
    Add-Content .\$filename "`"\LS:XmppFederationProxy - Streams\XmppFederationProxy - Failed inbound stream establishes/sec`""
    Add-Content .\$filename "`"\LS:XmppFederationProxy - Streams\XmppFederationProxy - Failed outbound stream establishes/sec`""
}

Function CreateSkype4BKHIConfigFile
{
    New-Item .\$filename -type file -Force | Out-Null
	
    #System Performance Counters
    Add-Content .\$filename "`"\Processor Information(*)\% Processor Time`""
    Add-Content .\$filename "`"\Memory\Available Mbytes`""
    Add-Content .\$filename "`"\Network Interface(*)\Output Queue Length`""
    Add-Content .\$filename "`"\Network Interface(*)\Outbound Packets Discarded`""
    Add-Content .\$filename "`"\Network Interface(*)\Inbound Packets Discarded`""
    Add-Content .\$filename "`"\Network Interface(*)\Packets Outbound Discarded`""
    Add-Content .\$filename "`"\Network Interface(*)\Packets Received Discarded`""
    Add-Content .\$filename "`"\PhysicalDisk(*)\Avg. Disk sec/Read`""
    Add-Content .\$filename "`"\PhysicalDisk(*)\Avg. Disk sec/Write`""

    #Front-End Server Counters
    Add-Content .\$filename "`"\LS:Usrv - DBStore\Usrv - Queue Latency (msec)`""
    Add-Content .\$filename "`"\LS:Usrv - DBStore\Usrv - Sproc Latency (msec)`""
    Add-Content .\$filename "`"\LS:Usrv - DBStore\Usrv - Throttled requests/sec`""
    Add-Content .\$filename "`"\LS:Usrv - REGDBStore\Usrv - Queue Latency (msec)`""
    Add-Content .\$filename "`"\LS:Usrv - REGDBStore\Usrv - Sproc Latency (msec)`""
    Add-Content .\$filename "`"\LS:Usrv - REGDBStore\Usrv - Throttled requests/sec`""
    Add-Content .\$filename "`"\LS:Usrv - SharedDBStore\Usrv - Queue Latency (msec)`""
    Add-Content .\$filename "`"\LS:Usrv - SharedDBStore\Usrv - Sproc Latency (msec)`""
    Add-Content .\$filename "`"\LS:Usrv - SharedDBStore\Usrv - Throttled requests/sec`""
    Add-Content .\$filename "`"\LS:SIP - Peers(*)\SIP - Sends Timed-Out /Sec`""
    Add-Content .\$filename "`"\LS:SIP - Responses\SIP - Local 503 Responses /Sec`""
    Add-Content .\$filename "`"\LS:SIP - Peers(*)\SIP - Average outgoing Queue Delay`""
    Add-Content .\$filename "`"\LS:SIP - Protocol\SIP - Incoming Requests Dropped /Sec`""
    Add-Content .\$filename "`"\LS:SIP - Peers(*)\SIP - Flow-controlled Connections`""
    Add-Content .\$filename "`"\LS:SIP - Protocol\SIP - Incoming Responses Dropped /Sec`""
    Add-Content .\$filename "`"\LS:SIP - Protocol\SIP - Average Incoming Message Processing Time`""
    Add-Content .\$filename "`"\LS:SIP - Load Management\SIP - Average Holding Time For Incoming Messages`""
    Add-Content .\$filename "`"\LS:SIP - Load Management\SIP - Incoming Messages Timed out`""
    Add-Content .\$filename "`"\LS:Usrv - Cluster Manager\Usrv - Number of failures of replication operations sent to other Replicas per second`""
    Add-Content .\$filename "`"\LS:XMPPFederation - SIP Instant Messaging\XMPPFederation - Failure IMDNs sent/sec`""
    Add-Content .\$filename "`"\LS:RoutingApps - Emergency Call Routing\RoutingApps - Number of incoming failure responses`""
    Add-Content .\$filename "`"\LS:LYSS - Storage Service API\LYSS - Current percentage of space used by Storage Service DB.`""

    #Logman will ignore Counters that don't exist on host, so we'll try BE with common instance names and FE instance names
    #SQL Server Counters
    Add-Content .\$filename "`"\MSSQL`$RTC:Buffer Manager\Page life expectancy`""    
    Add-Content .\$filename "`"\MSSQL`$MSSQLSERVER:Buffer Manager\Page life expectancy`""
    Add-Content .\$filename "`"\MSSQL`$RTCLOCAL:Buffer Manager\Page life expectancy`""
    Add-Content .\$filename "`"\MSSQL`$LYNCLOCAL:Buffer Manager\Page life expectancy`""

    #Front-End MCU Counters
    Add-Content .\$filename "`"\LS:DATAMCU - MCU Health And Performance\DATAMCU - MCU Health State`""
    Add-Content .\$filename "`"\LS:ASMCU - MCU Health And Performance\ASMCU - MCU Health State`""
    Add-Content .\$filename "`"\LS:AVMCU - MCU Health And Performance\AVMCU - MCU Health State`""
    Add-Content .\$filename "`"\LS:IMMCU - MCU Health And Performance\IMMCU - MCU Health State`""
    Add-Content .\$filename "`"\LS:IMMCU - IMMcu Conferences\IMMCU - Throttled Sip Connections`""

    #Front-End Web Services Counters
    Add-Content .\$filename "`"\ASP.NET Apps v4.0.30319(*)\Requests Rejected`""
    Add-Content .\$filename "`"\LS:WEB - Distribution List Expansion\WEB - Timed out Active Directory Requests/sec`""
    Add-Content .\$filename "`"\LS:WEB - Address Book File Download\WEB - Failed File Requests/Second`""
    Add-Content .\$filename "`"\LS:JoinLauncher - Join Launcher Service Failures\JOINLAUNCHER - Join Failures`""
    Add-Content .\$filename "`"\LS:WEB - UCWA\UCWA - HTTP 5xx Responses/Second`""
    Add-Content .\$filename "`"\LS:WEB - Auth Provider related calls\WEB - Failed validate cert calls to the cert auth provider`""
    Add-Content .\$filename "`"\LS:WEB - Address Book Web Query\WEB - Failed search requests/sec`""

    #Front-End Conferencing Services Counters
    Add-Content .\$filename "`"\LS:CAA - Operations\CAA - Incomplete calls per sec`""
    Add-Content .\$filename "`"\LS:USrv - Conference Control Notification\USrv - Notifications in processing`""
    Add-Content .\$filename "`"\LS:USrv - Conference Mcu Allocator\USrv - Factory Call Latency (msec)`""
    Add-Content .\$filename "`"\LS:USrv - Conference Mcu Allocator\USrv - Create Conference Latency (msec)`""
    Add-Content .\$filename "`"\LS:USrv - Conference Mcu Allocator\USrv - Allocation Latency (msec)`""

    #Mediation Server Counters
    Add-Content .\$filename "`"\LS:MediationServer - Media Relay\- Media Connectivity Check Failure`""
    Add-Content .\$filename "`"\LS:MediationServer - Health Indices\- Load Call Failure Index`""
    Add-Content .\$filename "`"\LS:MediationServer - Global Counters\- Total failed calls caused by unexpected interaction from the Proxy`""
    Add-Content .\$filename "`"\LS:MediationServer - Global Per Gateway Counters(*)\- Total failed calls caused by unexpected interaction from a gateway`""
    Add-Content .\$filename "`"\LS:MediationServer - Outbound Calls\-Total rejected due to load`""
    Add-Content .\$filename "`"\LS:MediationServer - Inbound Calls\-Total rejected due to load`""

    #Video Integration Server
    Add-Content .\$filename "`"\LS:VISSvc - VISSvc\VIS - Total Calls declined due to load`""
    Add-Content .\$filename "`"\LS:VISSvc - VISSvc\VIS - Total Proxy Leg Call Failures`""
    Add-Content .\$filename "`"\LS:VISSvc - VISSvc\VIS - Total Interop Leg Call Failures`""

    #Edge Server Counters
    Add-Content .\$filename "`"\LS:A/V Auth - Requests\- Bad Requests Received/sec`""
    Add-Content .\$filename "`"\LS:A/V Edge - UDP Counters(*)\A/V Edge - Authentication Failures/sec`""
    Add-Content .\$filename "`"\LS:A/V Edge - UDP Counters(*)\A/V Edge - Allocate Requests Exceeding Port Limit`""
    Add-Content .\$filename "`"\LS:A/V Edge - UDP Counters(*)\A/V Edge - Packets Dropped/sec`""
    Add-Content .\$filename "`"\LS:A/V Edge - TCP Counters(*)\A/V Edge - Authentication Failures/sec`""
    Add-Content .\$filename "`"\LS:A/V Edge - TCP Counters(*)\A/V Edge - Allocate Requests Exceeding Port Limit`""
    Add-Content .\$filename "`"\LS:A/V Edge - TCP Counters(*)\A/V Edge - Packets Dropped/sec`""
    Add-Content .\$filename "`"\LS:SIP - Peers(*)\SIP - Above Limit Connections Dropped (Access Proxies only)`""
    Add-Content .\$filename "`"\LS:DATAPROXY - Server Connections(*)\DATAPROXY - Current count of server connections that are throttled`""
    Add-Content .\$filename "`"\LS:DATAPROXY - Server Connections(*)\DATAPROXY - System is throttling`""
    Add-Content .\$filename "`"\LS:XmppFederationProxy - Streams\XmppFederationProxy - Failed inbound stream establishes/sec`""
    Add-Content .\$filename "`"\LS:XmppFederationProxy - Streams\XmppFederationProxy - Failed outbound stream establishes/sec`""
}

Function CreateSFB2019BKHIConfigFile
{
    New-Item .\$filename -type file -Force | Out-Null

    #System Performance Counters
    Add-Content .\$filename "`"\Processor Information(*)\% Processor Time`""
    Add-Content .\$filename "`"\Memory\Available Mbytes`""
    Add-Content .\$filename "`"\Network Interface(*)\Output Queue Length`""
    Add-Content .\$filename "`"\Network Interface(*)\Outbound Packets Discarded`""
    Add-Content .\$filename "`"\Network Interface(*)\Inbound Packets Discarded`""
    Add-Content .\$filename "`"\Network Interface(*)\Packets Outbound Discarded`""
    Add-Content .\$filename "`"\Network Interface(*)\Packets Received Discarded`""
    Add-Content .\$filename "`"\PhysicalDisk(*)\Avg. Disk sec/Read`""
    Add-Content .\$filename "`"\PhysicalDisk(*)\Avg. Disk sec/Write`""
    Add-Content .\$filename "`"\PhysicalDisk(*)\Avg. Disk Queue Length`""

    #Front-End Server Counters
    Add-Content .\$filename "`"\LS:Usrv - DBStore\Usrv - Queue Latency (msec)`""
    Add-Content .\$filename "`"\LS:Usrv - DBStore\Usrv - Sproc Latency (msec)`""
    Add-Content .\$filename "`"\LS:Usrv - DBStore\Usrv - Throttled requests/sec`""
    Add-Content .\$filename "`"\LS:Usrv - REGDBStore\Usrv - Queue Latency (msec)`""
    Add-Content .\$filename "`"\LS:Usrv - REGDBStore\Usrv - Sproc Latency (msec)`""
    Add-Content .\$filename "`"\LS:Usrv - REGDBStore\Usrv - Throttled requests/sec`""
    Add-Content .\$filename "`"\LS:Usrv - SharedDBStore\Usrv - Queue Latency (msec)`""
    Add-Content .\$filename "`"\LS:Usrv - SharedDBStore\Usrv - Sproc Latency (msec)`""
    Add-Content .\$filename "`"\LS:Usrv - SharedDBStore\Usrv - Throttled requests/sec`""
    Add-Content .\$filename "`"\LS:UPINSVC - UserPinDBAccess\UPINSVC - Sproc Latency (msec)`""
    Add-Content .\$filename "`"\LS:UPINSVC - UserPinDBAccess\UPINSVC - Queue Latency (msec)`""
    Add-Content .\$filename "`"\LS:SIP - Peers(*)\SIP - Sends Timed-Out /Sec`""
    Add-Content .\$filename "`"\LS:SIP - Responses\SIP - Local 503 Responses /Sec`""
    Add-Content .\$filename "`"\LS:SIP - Peers(*)\SIP - Average outgoing Queue Delay`""
    Add-Content .\$filename "`"\LS:SIP - Protocol\SIP - Incoming Requests Dropped /Sec`""
    Add-Content .\$filename "`"\LS:SIP - Peers(*)\SIP - Flow-controlled Connections`""
    Add-Content .\$filename "`"\LS:SIP - Protocol\SIP - Incoming Responses Dropped /Sec`""
    Add-Content .\$filename "`"\LS:SIP - Protocol\SIP - Average Incoming Message Processing Time`""
    Add-Content .\$filename "`"\LS:SIP - Load Management\SIP - Average Holding Time For Incoming Messages`""
    Add-Content .\$filename "`"\LS:SIP - Load Management\SIP - Incoming Messages Timed out`""
    Add-Content .\$filename "`"\LS:Usrv - Cluster Manager\Usrv - Number of failures of replication operations sent to other Replicas per second`""
    Add-Content .\$filename "`"\LS:RoutingApps - Emergency Call Routing\RoutingApps - Number of incoming failure responses`""
    Add-Content .\$filename "`"\LS:LYSS - Storage Service API\LYSS - Current percentage of space used by Storage Service DB.`""

    #Logman will ignore Counters that don't exist on host, so we'll try BE with common instance names and FE instance names
    #SQL Server Counters
    Add-Content .\$filename "`"\MSSQL`$RTC:Buffer Manager\Page life expectancy`""    
    Add-Content .\$filename "`"\MSSQL`$MSSQLSERVER:Buffer Manager\Page life expectancy`""
    Add-Content .\$filename "`"\MSSQL`$RTCLOCAL:Buffer Manager\Page life expectancy`""
    Add-Content .\$filename "`"\MSSQL`$LYNCLOCAL:Buffer Manager\Page life expectancy`""

    #Front-End MCU Counters
    Add-Content .\$filename "`"\LS:DATAMCU - MCU Health And Performance\DATAMCU - MCU Health State`""
    Add-Content .\$filename "`"\LS:ASMCU - MCU Health And Performance\ASMCU - MCU Health State`""
    Add-Content .\$filename "`"\LS:AVMCU - MCU Health And Performance\AVMCU - MCU Health State`""
    Add-Content .\$filename "`"\LS:IMMCU - MCU Health And Performance\IMMCU - MCU Health State`""
    Add-Content .\$filename "`"\LS:IMMCU - IMMcu Conferences\IMMCU - Throttled Sip Connections`""

    #Front-End Web Services Counters
    Add-Content .\$filename "`"\ASP.NET Apps v4.0.30319(*)\Requests Rejected`""
    Add-Content .\$filename "`"\LS:WEB - Distribution List Expansion\WEB - Timed out Active Directory Requests/sec`""
    Add-Content .\$filename "`"\LS:WEB - Address Book File Download\WEB - Failed File Requests/Second`""
    Add-Content .\$filename "`"\LS:JoinLauncher - Join Launcher Service Failures\JOINLAUNCHER - Join Failures`""
    Add-Content .\$filename "`"\LS:WEB - UCWA(_Total)\UCWA - HTTP 5xx Responses/Second`""
    Add-Content .\$filename "`"\LS:WEB - UCWA(_Total)\UCWA - Number of task queues exceeding the queue depth limit`""
    Add-Content .\$filename "`"\LS:WEB - Auth Provider related calls\WEB - Failed validate cert calls to the cert auth provider`""
    Add-Content .\$filename "`"\LS:WEB - Address Book Web Query\WEB - Failed search requests/sec`""

    #Front-End Conferencing Services Counters
    Add-Content .\$filename "`"\LS:CAA - Operations\CAA - Incomplete calls per sec`""
    Add-Content .\$filename "`"\LS:USrv - Conference Control Notification\USrv - Notifications in processing`""
    Add-Content .\$filename "`"\LS:USrv - Conference Mcu Allocator\USrv - Factory Call Latency (msec)`""
    Add-Content .\$filename "`"\LS:USrv - Conference Mcu Allocator\USrv - Create Conference Latency (msec)`""
    Add-Content .\$filename "`"\LS:USrv - Conference Mcu Allocator\USrv - Allocation Latency (msec)`""
    Add-Content .\$filename "`"\LS:USrv - Notify\USrv - Notification Queue Highest Queue Latency`""
    Add-Content .\$filename "`"\LS:USrv - Conference Focus Factory\USrv - Add conference request latency (msec)`""
    Add-Content .\$filename "`"\LS:USrv - Conference Focus Base\USrv - Join INVITE latency (msec)`""
    Add-Content .\$filename "`"\LS:USrv - Conference Focus Base\USrv - Media INVITE latency (msec)`""

    #Mediation Server Counters
    Add-Content .\$filename "`"\LS:MediationServer - Media Relay\- Media Connectivity Check Failure`""
    Add-Content .\$filename "`"\LS:MediationServer - Health Indices\- Load Call Failure Index`""
    Add-Content .\$filename "`"\LS:MediationServer - Global Counters\- Total failed calls caused by unexpected interaction from the Proxy`""
    Add-Content .\$filename "`"\LS:MediationServer - Global Per Gateway Counters(*)\- Total failed calls caused by unexpected interaction from a gateway`""
    Add-Content .\$filename "`"\LS:MediationServer - Outbound Calls\-Total rejected due to load`""
    Add-Content .\$filename "`"\LS:MediationServer - Inbound Calls\-Total rejected due to load`""

    #Video Integration Server
    Add-Content .\$filename "`"\LS:VISSvc - VISSvc\VIS - Total Calls declined due to load`""
    Add-Content .\$filename "`"\LS:VISSvc - VISSvc\VIS - Total Proxy Leg Call Failures`""
    Add-Content .\$filename "`"\LS:VISSvc - VISSvc\VIS - Total Interop Leg Call Failures`""

    #Edge Server Counters
    Add-Content .\$filename "`"\LS:A/V Auth - Requests\- Bad Requests Received/sec`""
    Add-Content .\$filename "`"\LS:A/V Edge - UDP Counters(*)\A/V Edge - Authentication Failures/sec`""
    Add-Content .\$filename "`"\LS:A/V Edge - UDP Counters(*)\A/V Edge - Allocate Requests Exceeding Port Limit`""
    Add-Content .\$filename "`"\LS:A/V Edge - UDP Counters(*)\A/V Edge - Packets Dropped/sec`""
    Add-Content .\$filename "`"\LS:A/V Edge - TCP Counters(*)\A/V Edge - Authentication Failures/sec`""
    Add-Content .\$filename "`"\LS:A/V Edge - TCP Counters(*)\A/V Edge - Allocate Requests Exceeding Port Limit`""
    Add-Content .\$filename "`"\LS:A/V Edge - TCP Counters(*)\A/V Edge - Packets Dropped/sec`""
    Add-Content .\$filename "`"\LS:SIP - Peers(*)\SIP - Above Limit Connections Dropped (Access Proxies only)`""
    Add-Content .\$filename "`"\LS:DATAPROXY - Server Connections(*)\DATAPROXY - Current count of server connections that are throttled`""
    Add-Content .\$filename "`"\LS:DATAPROXY - Server Connections(*)\DATAPROXY - System is throttling`""
}

Function AddSQLInstance($NewInstance)
{
    Add-Content .\$filename "`"\MSSQL`$$NewInstance`:Buffer Manager\Page life expectancy`""
}

If (-NOT ([Security.Principal.WindowsPrincipal] [Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole([Security.Principal.WindowsBuiltInRole] "Administrator"))
{
    Write-Host -ForegroundColor Red "Error: Insufficent Rights. Please launch Powershell as Administrator and retry."
    Exit
}

If ($Computer -eq "") {
    $Computer = $env:COMPUTERNAME
}

If ($Version -eq "LyncServer2010")
{
    CreateLyncServer2010KHIConfigFile
}

If ($Version -eq "LyncServer2013")
{
    CreateLyncServer2013KHIConfigFile
}

If ($Version -eq "Skype4B")
{
    CreateSkype4BKHIConfigFile
}

If ($Version -eq "SFB2019")
{
    CreateSFB2019BKHIConfigFile
}

If ($SQLInstance -ne "")
{
    AddSQLInstance($SQLInstance)
}

CreateDataCollector($Computer.ToUpper())
Remove-Item .\KHI.config


# SIG # Begin signature block
# MIIdkgYJKoZIhvcNAQcCoIIdgzCCHX8CAQExCzAJBgUrDgMCGgUAMGkGCisGAQQB
# gjcCAQSgWzBZMDQGCisGAQQBgjcCAR4wJgIDAQAABBAfzDtgWUsITrck0sYpfvNR
# AgEAAgEAAgEAAgEAAgEAMCEwCQYFKw4DAhoFAAQUz5Z7a9Ps9EBbQtiCLQyj5/YW
# K9OgghhuMIIE3jCCA8agAwIBAgITMwAAAPfdvzTg5NWCYAAAAAAA9zANBgkqhkiG
# 9w0BAQUFADB3MQswCQYDVQQGEwJVUzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4G
# A1UEBxMHUmVkbW9uZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSEw
# HwYDVQQDExhNaWNyb3NvZnQgVGltZS1TdGFtcCBQQ0EwHhcNMTgwODIzMjAyMDAy
# WhcNMTkxMTIzMjAyMDAyWjCBzjELMAkGA1UEBhMCVVMxEzARBgNVBAgTCldhc2hp
# bmd0b24xEDAOBgNVBAcTB1JlZG1vbmQxHjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jw
# b3JhdGlvbjEpMCcGA1UECxMgTWljcm9zb2Z0IE9wZXJhdGlvbnMgUHVlcnRvIFJp
# Y28xJjAkBgNVBAsTHVRoYWxlcyBUU1MgRVNOOjdEMkUtMzc4Mi1CMEY3MSUwIwYD
# VQQDExxNaWNyb3NvZnQgVGltZS1TdGFtcCBTZXJ2aWNlMIIBIjANBgkqhkiG9w0B
# AQEFAAOCAQ8AMIIBCgKCAQEA3nEYJOthhViLNfJ3TBvlYEfBf7ap9sFWO/VXhvxg
# oNT6yJT2ZJjY/WbvzyYr4eaV6xqRUr0WM+sYmOaHioAKaoVRm3rTboJa+ggffCou
# AAX2MwVp41p3ojfe2HTnAUSiw+G28J6eyggABXmPSbAl0Y7kjibEEnVjNIK5ycYz
# 4B0CefTmxi7LKfTL4JYpyP9IXH1BjUDZ4VszdvN+57LDPc2Wsf5kGGTVizX7znqv
# 99TSoldE0kilSZyfwotZcFRfObsImAYH5r6eMMuC2kJR5kYUCWkt7W5gSZ/wqAL3
# tEEbkkRR561DjwjfgAY/8CILNRr5NoPCyj2fgr2wlxZeGwIDAQABo4IBCTCCAQUw
# HQYDVR0OBBYEFFkoL/rgdU0f1ZHJFNOCZplml0/bMB8GA1UdIwQYMBaAFCM0+NlS
# RnAK7UD7dvuzK7DDNbMPMFQGA1UdHwRNMEswSaBHoEWGQ2h0dHA6Ly9jcmwubWlj
# cm9zb2Z0LmNvbS9wa2kvY3JsL3Byb2R1Y3RzL01pY3Jvc29mdFRpbWVTdGFtcFBD
# QS5jcmwwWAYIKwYBBQUHAQEETDBKMEgGCCsGAQUFBzAChjxodHRwOi8vd3d3Lm1p
# Y3Jvc29mdC5jb20vcGtpL2NlcnRzL01pY3Jvc29mdFRpbWVTdGFtcFBDQS5jcnQw
# EwYDVR0lBAwwCgYIKwYBBQUHAwgwDQYJKoZIhvcNAQEFBQADggEBAGLhSXaU0PCt
# JC8w07NjC/pFMxBbsAr9v0Vx5PFm+R9sglray9H7RCDOc+snob0MdTpqPwHavJqW
# PrnI/zwWX7C5gb4GtarS08hcTDPEeqCxCQWCdmI1WB5ReZRjcEu4/3Yt0kldEdor
# v94tu4VNPdHEK54CJ+Zaz7KPEBJNDKW5gUh21Po1nx4f1oIATChhmTGoRJdGi5pO
# VM+P+LTpAEAD1RwWTNHB1q7ofM3Mwb7q0v81TsDOtXqtu6a7LtyU6fMefeDsKKMH
# gmZaw1tay+wDyeMslBUhK6D52ZtL57be6yBRjD76LCPVGLDwsaBwbHrvi4NIpjoC
# Bu+giiy3iEUwggX/MIID56ADAgECAhMzAAABA14lHJkfox64AAAAAAEDMA0GCSqG
# SIb3DQEBCwUAMH4xCzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpXYXNoaW5ndG9uMRAw
# DgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVNaWNyb3NvZnQgQ29ycG9yYXRpb24x
# KDAmBgNVBAMTH01pY3Jvc29mdCBDb2RlIFNpZ25pbmcgUENBIDIwMTEwHhcNMTgw
# NzEyMjAwODQ4WhcNMTkwNzI2MjAwODQ4WjB0MQswCQYDVQQGEwJVUzETMBEGA1UE
# CBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwGA1UEChMVTWljcm9z
# b2Z0IENvcnBvcmF0aW9uMR4wHAYDVQQDExVNaWNyb3NvZnQgQ29ycG9yYXRpb24w
# ggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAwggEKAoIBAQDRlHY25oarNv5p+UZ8i4hQ
# y5Bwf7BVqSQdfjnnBZ8PrHuXss5zCvvUmyRcFrU53Rt+M2wR/Dsm85iqXVNrqsPs
# E7jS789Xf8xly69NLjKxVitONAeJ/mkhvT5E+94SnYW/fHaGfXKxdpth5opkTEbO
# ttU6jHeTd2chnLZaBl5HhvU80QnKDT3NsumhUHjRhIjiATwi/K+WCMxdmcDt66Va
# mJL1yEBOanOv3uN0etNfRpe84mcod5mswQ4xFo8ADwH+S15UD8rEZT8K46NG2/Ys
# AzoZvmgFFpzmfzS/p4eNZTkmyWPU78XdvSX+/Sj0NIZ5rCrVXzCRO+QUauuxygQj
# AgMBAAGjggF+MIIBejAfBgNVHSUEGDAWBgorBgEEAYI3TAgBBggrBgEFBQcDAzAd
# BgNVHQ4EFgQUR77Ay+GmP/1l1jjyA123r3f3QP8wUAYDVR0RBEkwR6RFMEMxKTAn
# BgNVBAsTIE1pY3Jvc29mdCBPcGVyYXRpb25zIFB1ZXJ0byBSaWNvMRYwFAYDVQQF
# Ew0yMzAwMTIrNDM3OTY1MB8GA1UdIwQYMBaAFEhuZOVQBdOCqhc3NyK1bajKdQKV
# MFQGA1UdHwRNMEswSaBHoEWGQ2h0dHA6Ly93d3cubWljcm9zb2Z0LmNvbS9wa2lv
# cHMvY3JsL01pY0NvZFNpZ1BDQTIwMTFfMjAxMS0wNy0wOC5jcmwwYQYIKwYBBQUH
# AQEEVTBTMFEGCCsGAQUFBzAChkVodHRwOi8vd3d3Lm1pY3Jvc29mdC5jb20vcGtp
# b3BzL2NlcnRzL01pY0NvZFNpZ1BDQTIwMTFfMjAxMS0wNy0wOC5jcnQwDAYDVR0T
# AQH/BAIwADANBgkqhkiG9w0BAQsFAAOCAgEAn/XJUw0/DSbsokTYDdGfY5YGSz8e
# XMUzo6TDbK8fwAG662XsnjMQD6esW9S9kGEX5zHnwya0rPUn00iThoj+EjWRZCLR
# ay07qCwVlCnSN5bmNf8MzsgGFhaeJLHiOfluDnjYDBu2KWAndjQkm925l3XLATut
# ghIWIoCJFYS7mFAgsBcmhkmvzn1FFUM0ls+BXBgs1JPyZ6vic8g9o838Mh5gHOmw
# GzD7LLsHLpaEk0UoVFzNlv2g24HYtjDKQ7HzSMCyRhxdXnYqWJ/U7vL0+khMtWGL
# sIxB6aq4nZD0/2pCD7k+6Q7slPyNgLt44yOneFuybR/5WcF9ttE5yXnggxxgCto9
# sNHtNr9FB+kbNm7lPTsFA6fUpyUSj+Z2oxOzRVpDMYLa2ISuubAfdfX2HX1RETcn
# 6LU1hHH3V6qu+olxyZjSnlpkdr6Mw30VapHxFPTy2TUxuNty+rR1yIibar+YRcdm
# stf/zpKQdeTr5obSyBvbJ8BblW9Jb1hdaSreU0v46Mp79mwV+QMZDxGFqk+av6pX
# 3WDG9XEg9FGomsrp0es0Rz11+iLsVT9qGTlrEOlaP470I3gwsvKmOMs1jaqYWSRA
# uDpnpAdfoP7YO0kT+wzh7Qttg1DO8H8+4NkI6IwhSkHC3uuOW+4Dwx1ubuZUNWZn
# cnwa6lL2IsRyP64wggYHMIID76ADAgECAgphFmg0AAAAAAAcMA0GCSqGSIb3DQEB
# BQUAMF8xEzARBgoJkiaJk/IsZAEZFgNjb20xGTAXBgoJkiaJk/IsZAEZFgltaWNy
# b3NvZnQxLTArBgNVBAMTJE1pY3Jvc29mdCBSb290IENlcnRpZmljYXRlIEF1dGhv
# cml0eTAeFw0wNzA0MDMxMjUzMDlaFw0yMTA0MDMxMzAzMDlaMHcxCzAJBgNVBAYT
# AlVTMRMwEQYDVQQIEwpXYXNoaW5ndG9uMRAwDgYDVQQHEwdSZWRtb25kMR4wHAYD
# VQQKExVNaWNyb3NvZnQgQ29ycG9yYXRpb24xITAfBgNVBAMTGE1pY3Jvc29mdCBU
# aW1lLVN0YW1wIFBDQTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAJ+h
# bLHf20iSKnxrLhnhveLjxZlRI1Ctzt0YTiQP7tGn0UytdDAgEesH1VSVFUmUG0KS
# rphcMCbaAGvoe73siQcP9w4EmPCJzB/LMySHnfL0Zxws/HvniB3q506jocEjU8qN
# +kXPCdBer9CwQgSi+aZsk2fXKNxGU7CG0OUoRi4nrIZPVVIM5AMs+2qQkDBuh/NZ
# MJ36ftaXs+ghl3740hPzCLdTbVK0RZCfSABKR2YRJylmqJfk0waBSqL5hKcRRxQJ
# gp+E7VV4/gGaHVAIhQAQMEbtt94jRrvELVSfrx54QTF3zJvfO4OToWECtR0Nsfz3
# m7IBziJLVP/5BcPCIAsCAwEAAaOCAaswggGnMA8GA1UdEwEB/wQFMAMBAf8wHQYD
# VR0OBBYEFCM0+NlSRnAK7UD7dvuzK7DDNbMPMAsGA1UdDwQEAwIBhjAQBgkrBgEE
# AYI3FQEEAwIBADCBmAYDVR0jBIGQMIGNgBQOrIJgQFYnl+UlE/wq4QpTlVnkpKFj
# pGEwXzETMBEGCgmSJomT8ixkARkWA2NvbTEZMBcGCgmSJomT8ixkARkWCW1pY3Jv
# c29mdDEtMCsGA1UEAxMkTWljcm9zb2Z0IFJvb3QgQ2VydGlmaWNhdGUgQXV0aG9y
# aXR5ghB5rRahSqClrUxzWPQHEy5lMFAGA1UdHwRJMEcwRaBDoEGGP2h0dHA6Ly9j
# cmwubWljcm9zb2Z0LmNvbS9wa2kvY3JsL3Byb2R1Y3RzL21pY3Jvc29mdHJvb3Rj
# ZXJ0LmNybDBUBggrBgEFBQcBAQRIMEYwRAYIKwYBBQUHMAKGOGh0dHA6Ly93d3cu
# bWljcm9zb2Z0LmNvbS9wa2kvY2VydHMvTWljcm9zb2Z0Um9vdENlcnQuY3J0MBMG
# A1UdJQQMMAoGCCsGAQUFBwMIMA0GCSqGSIb3DQEBBQUAA4ICAQAQl4rDXANENt3p
# tK132855UU0BsS50cVttDBOrzr57j7gu1BKijG1iuFcCy04gE1CZ3XpA4le7r1ia
# HOEdAYasu3jyi9DsOwHu4r6PCgXIjUji8FMV3U+rkuTnjWrVgMHmlPIGL4UD6ZEq
# JCJw+/b85HiZLg33B+JwvBhOnY5rCnKVuKE5nGctxVEO6mJcPxaYiyA/4gcaMvnM
# MUp2MT0rcgvI6nA9/4UKE9/CCmGO8Ne4F+tOi3/FNSteo7/rvH0LQnvUU3Ih7jDK
# u3hlXFsBFwoUDtLaFJj1PLlmWLMtL+f5hYbMUVbonXCUbKw5TNT2eb+qGHpiKe+i
# myk0BncaYsk9Hm0fgvALxyy7z0Oz5fnsfbXjpKh0NbhOxXEjEiZ2CzxSjHFaRkMU
# vLOzsE1nyJ9C/4B5IYCeFTBm6EISXhrIniIh0EPpK+m79EjMLNTYMoBMJipIJF9a
# 6lbvpt6Znco6b72BJ3QGEe52Ib+bgsEnVLaxaj2JoXZhtG6hE6a/qkfwEm/9ijJs
# sv7fUciMI8lmvZ0dhxJkAj0tr1mPuOQh5bWwymO0eFQF1EEuUKyUsKV4q7OglnUa
# 2ZKHE3UiLzKoCG6gW4wlv6DvhMoh1useT8ma7kng9wFlb4kLfchpyOZu6qeXzjEp
# /w7FW1zYTRuh2Povnj8uVRZryROj/TCCB3owggVioAMCAQICCmEOkNIAAAAAAAMw
# DQYJKoZIhvcNAQELBQAwgYgxCzAJBgNVBAYTAlVTMRMwEQYDVQQIEwpXYXNoaW5n
# dG9uMRAwDgYDVQQHEwdSZWRtb25kMR4wHAYDVQQKExVNaWNyb3NvZnQgQ29ycG9y
# YXRpb24xMjAwBgNVBAMTKU1pY3Jvc29mdCBSb290IENlcnRpZmljYXRlIEF1dGhv
# cml0eSAyMDExMB4XDTExMDcwODIwNTkwOVoXDTI2MDcwODIxMDkwOVowfjELMAkG
# A1UEBhMCVVMxEzARBgNVBAgTCldhc2hpbmd0b24xEDAOBgNVBAcTB1JlZG1vbmQx
# HjAcBgNVBAoTFU1pY3Jvc29mdCBDb3Jwb3JhdGlvbjEoMCYGA1UEAxMfTWljcm9z
# b2Z0IENvZGUgU2lnbmluZyBQQ0EgMjAxMTCCAiIwDQYJKoZIhvcNAQEBBQADggIP
# ADCCAgoCggIBAKvw+nIQHC6t2G6qghBNNLrytlghn0IbKmvpWlCquAY4GgRJun/D
# DB7dN2vGEtgL8DjCmQawyDnVARQxQtOJDXlkh36UYCRsr55JnOloXtLfm1OyCizD
# r9mpK656Ca/XllnKYBoF6WZ26DJSJhIv56sIUM+zRLdd2MQuA3WraPPLbfM6XKEW
# 9Ea64DhkrG5kNXimoGMPLdNAk/jj3gcN1Vx5pUkp5w2+oBN3vpQ97/vjK1oQH01W
# KKJ6cuASOrdJXtjt7UORg9l7snuGG9k+sYxd6IlPhBryoS9Z5JA7La4zWMW3Pv4y
# 07MDPbGyr5I4ftKdgCz1TlaRITUlwzluZH9TupwPrRkjhMv0ugOGjfdf8NBSv4yU
# h7zAIXQlXxgotswnKDglmDlKNs98sZKuHCOnqWbsYR9q4ShJnV+I4iVd0yFLPlLE
# tVc/JAPw0XpbL9Uj43BdD1FGd7P4AOG8rAKCX9vAFbO9G9RVS+c5oQ/pI0m8GLhE
# fEXkwcNyeuBy5yTfv0aZxe/CHFfbg43sTUkwp6uO3+xbn6/83bBm4sGXgXvt1u1L
# 50kppxMopqd9Z4DmimJ4X7IvhNdXnFy/dygo8e1twyiPLI9AN0/B4YVEicQJTMXU
# pUMvdJX3bvh4IFgsE11glZo+TzOE2rCIF96eTvSWsLxGoGyY0uDWiIwLAgMBAAGj
# ggHtMIIB6TAQBgkrBgEEAYI3FQEEAwIBADAdBgNVHQ4EFgQUSG5k5VAF04KqFzc3
# IrVtqMp1ApUwGQYJKwYBBAGCNxQCBAweCgBTAHUAYgBDAEEwCwYDVR0PBAQDAgGG
# MA8GA1UdEwEB/wQFMAMBAf8wHwYDVR0jBBgwFoAUci06AjGQQ7kUBU7h6qfHMdEj
# iTQwWgYDVR0fBFMwUTBPoE2gS4ZJaHR0cDovL2NybC5taWNyb3NvZnQuY29tL3Br
# aS9jcmwvcHJvZHVjdHMvTWljUm9vQ2VyQXV0MjAxMV8yMDExXzAzXzIyLmNybDBe
# BggrBgEFBQcBAQRSMFAwTgYIKwYBBQUHMAKGQmh0dHA6Ly93d3cubWljcm9zb2Z0
# LmNvbS9wa2kvY2VydHMvTWljUm9vQ2VyQXV0MjAxMV8yMDExXzAzXzIyLmNydDCB
# nwYDVR0gBIGXMIGUMIGRBgkrBgEEAYI3LgMwgYMwPwYIKwYBBQUHAgEWM2h0dHA6
# Ly93d3cubWljcm9zb2Z0LmNvbS9wa2lvcHMvZG9jcy9wcmltYXJ5Y3BzLmh0bTBA
# BggrBgEFBQcCAjA0HjIgHQBMAGUAZwBhAGwAXwBwAG8AbABpAGMAeQBfAHMAdABh
# AHQAZQBtAGUAbgB0AC4gHTANBgkqhkiG9w0BAQsFAAOCAgEAZ/KGpZjgVHkaLtPY
# dGcimwuWEeFjkplCln3SeQyQwWVfLiw++MNy0W2D/r4/6ArKO79HqaPzadtjvyI1
# pZddZYSQfYtGUFXYDJJ80hpLHPM8QotS0LD9a+M+By4pm+Y9G6XUtR13lDni6WTJ
# RD14eiPzE32mkHSDjfTLJgJGKsKKELukqQUMm+1o+mgulaAqPyprWEljHwlpblqY
# luSD9MCP80Yr3vw70L01724lruWvJ+3Q3fMOr5kol5hNDj0L8giJ1h/DMhji8MUt
# zluetEk5CsYKwsatruWy2dsViFFFWDgycScaf7H0J/jeLDogaZiyWYlobm+nt3TD
# QAUGpgEqKD6CPxNNZgvAs0314Y9/HG8VfUWnduVAKmWjw11SYobDHWM2l4bf2vP4
# 8hahmifhzaWX0O5dY0HjWwechz4GdwbRBrF1HxS+YWG18NzGGwS+30HHDiju3mUv
# 7Jf2oVyW2ADWoUa9WfOXpQlLSBCZgB/QACnFsZulP0V3HjXG0qKin3p6IvpIlR+r
# +0cjgPWe+L9rt0uX4ut1eBrs6jeZeRhL/9azI2h15q/6/IvrC4DqaTuv/DDtBEyO
# 3991bWORPdGdVk5Pv4BXIqF4ETIheu9BCrE/+6jMpF3BoYibV3FWTkhFwELJm3Zb
# CoBIa/15n8G9bW1qyVJzEw16UM0xggSOMIIEigIBATCBlTB+MQswCQYDVQQGEwJV
# UzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9uZDEeMBwGA1UE
# ChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSgwJgYDVQQDEx9NaWNyb3NvZnQgQ29k
# ZSBTaWduaW5nIFBDQSAyMDExAhMzAAABA14lHJkfox64AAAAAAEDMAkGBSsOAwIa
# BQCggaIwGQYJKoZIhvcNAQkDMQwGCisGAQQBgjcCAQQwHAYKKwYBBAGCNwIBCzEO
# MAwGCisGAQQBgjcCARUwIwYJKoZIhvcNAQkEMRYEFOTDGIDh6xO+SVHSUUa1Bzbq
# uRTrMEIGCisGAQQBgjcCAQwxNDAyoBSAEgBNAGkAYwByAG8AcwBvAGYAdKEagBho
# dHRwOi8vd3d3Lm1pY3Jvc29mdC5jb20wDQYJKoZIhvcNAQEBBQAEggEAb1g/dxSv
# 2Pd9uYItn0LAdWjGi2in+og3nIv6Xtck3Rq8obZ7IiqPLKxJNUTfbMdsiCuowFz1
# LP8JwZd9a/RC5LdoK14yssVIhdaLm3Hw5P5FLzkPqIz9hnO76gtzOdy44zv0i8+a
# 0JzMNWQ02OLaeObrdntmphAswRc95kJGERXTXSPiTVPyZH7NUK3cCfrz1JTN4UOl
# 2Txu3oRE77XqWwCpklJA8qRycfjBgDbnOdz47THG0IJ5V2CVImXdhNwuWuyChgeR
# a4mBZLMqeAVSpUNiZAQn3rxFyDH3ukR58+CpfRB2mHpl7ySPbemN65ZKcAEi3/mU
# 7XkmpBhFFzTOBaGCAigwggIkBgkqhkiG9w0BCQYxggIVMIICEQIBATCBjjB3MQsw
# CQYDVQQGEwJVUzETMBEGA1UECBMKV2FzaGluZ3RvbjEQMA4GA1UEBxMHUmVkbW9u
# ZDEeMBwGA1UEChMVTWljcm9zb2Z0IENvcnBvcmF0aW9uMSEwHwYDVQQDExhNaWNy
# b3NvZnQgVGltZS1TdGFtcCBQQ0ECEzMAAAD33b804OTVgmAAAAAAAPcwCQYFKw4D
# AhoFAKBdMBgGCSqGSIb3DQEJAzELBgkqhkiG9w0BBwEwHAYJKoZIhvcNAQkFMQ8X
# DTE4MTEwODE5MjMwMVowIwYJKoZIhvcNAQkEMRYEFBe4aZOzvRh92LF3dLCbM1OI
# gI0JMA0GCSqGSIb3DQEBBQUABIIBABNj5GktSXTrHHhds0Jp/cyB+plZ6qhipAwd
# 6pu592VuOMfFj9SH76N5GPiaQWoBYUlsVff8+H5ya7G7mH+KSLdWEDkFS4ZMERKS
# 9cz2sO10fKfJfuctpXuzj8jTMOTRbkpcld/vSEPoZCZYdo6r5e7j/B2d7DXMIdJo
# y1ArsPE57of4Yqp8sBDiSpo6XR+0Yn+OtNyvJK4KK0eeHg9aVCtGPKxyUjJR/xLK
# KKU/DFRC9cvwwzE47vGQNDZWsaO4PBQldRQR3VJt/PV7S6B0yIZ823sM3IE64HvP
# T8NBf2M7bQzEUPIY2h0D04LXbKxtvHpDZucUoUE9wvqdlzrkaG4=
# SIG # End signature block
